<?php

namespace KnitPay\Extensions\LearnPress;

use Pronamic\WordPress\Pay\Plugin;
use LP_Gateway_Abstract;
use LP_Settings;

/**
 * Title: Learn Press extension
 * Description:
 * Copyright: 2020 Knit Pay
 * Company: Knit Pay
 *
 * @author  knitpay
 * @since   1.6.0
 */

/**
 * Prevent loading this file directly
 */
defined( 'ABSPATH' ) || exit();
 
class Gateway extends LP_Gateway_Abstract{
    /**
     * @var LP_Settings
     */
    public $settings;

		/**
		 * @var string
		 */
		public $id = 'knit_pay';

	/**
	 * Admin label.
	 *
	 * @var string
	 */
	private $admin_label;

	/**
	 * Checkout label.
	 *
	 * @var string
	 */
	private $checkout_label;

	/**
	 * Payment method.
	 *
	 * @var string
	 */
	private $payment_method;

	/**
	 * Supports.
	 *
	 * @var array
	 */
	private $supports;

	/**
	 * Bootstrap
	 *
	 * @param array $args Gateway properties.
	 */
	public function __construct( /* $args */ ) {
	    parent::__construct();
	    $this->init();

	    $this->id = 'knit_pay';
	    
	    $this->method_title       = __( 'Knit Pay', 'learnpress' );
	    $this->method_description = __( "This payment method does not use a predefined payment method for the payment. Some payment providers list all activated payment methods for your account to choose from. Use payment method specific gateways (such as 'Instamojo') to let customers choose their desired payment method at checkout.", 'learnpress' );
	    $this->icon               = '';
	    
	    $this->title       = $this->settings->get('title')?$this->settings->get('title'):__( 'Knit Pay', 'learnpress' );
	    $this->description = $this->settings->get('description')?$this->settings->get('description'):__( 'Pay with Knit Pay', 'learnpress' );

	}
	
	public function get_settings() {
	    
	    return apply_filters( 'learn-press/gateway-payment/' . $this->id . '/settings',
	        array(
	            array(
	                'title'   => __( 'Enable', 'learnpress-knitpay-payment' ),
	                'id'      => '[enable]',
	                'default' => 'no',
	                'type'    => 'yes-no'
	            ),
	            array(
	                'title'      => __( 'Title', 'learnpress-knitpay-payment' ),
	                'id'         => '[title]',
	                'default'    => $this->title,
	                'type'       => 'text',
	                'visibility' => array(
	                    'state'       => 'show',
	                    'conditional' => array(
	                        array(
	                            'field'   => '[enable]',
	                            'compare' => '=',
	                            'value'   => 'yes'
	                        ),
	                        array(
	                            'field'   => '[test_mode]',
	                            'compare' => '!=',
	                            'value'   => 'yes'
	                        )
	                    )
	                )
	            ),
	            array(
	                'title'      => __( 'Instruction', 'learnpress-knitpay-payment' ),
	                'id'         => '[description]',
	                'default'    => $this->description,
	                'type'       => 'textarea',
	                'editor'     => array( 'textarea_rows' => 5 ),
	                'visibility' => array(
	                    'state'       => 'show',
	                    'conditional' => array(
	                        array(
	                            'field'   => '[enable]',
	                            'compare' => '=',
	                            'value'   => 'yes'
	                        ),
	                        array(
	                            'field'   => '[test_mode]',
	                            'compare' => '!=',
	                            'value'   => 'yes'
	                        )
	                    )
	                )
	            ),
	            array(
	                'title'   => __( 'Configuration', 'learnpress' ),
	                'id'      => '[config_id]',
	                'default' => get_option( 'pronamic_pay_config_id' ),
	                'type'    => 'select',
	                'options' => Plugin::get_config_select_options( $this->payment_method ),
	                'visibility' => array(
	                    'state'       => 'show',
	                    'conditional' => array(
	                        array(
	                            'field'   => '[enable]',
	                            'compare' => '=',
	                            'value'   => 'yes'
	                        ),
	                        array(
	                            'field'   => '[test_mode]',
	                            'compare' => '!=',
	                            'value'   => 'yes'
	                        )
	                    )
	                ),
	            ),
	            array(
	                'title'      => __( 'Payment Description', 'learnpress-knitpay-payment' ),
	                'id'         => '[payment_description]',
	                'default'    => __( 'Learn Press Order {order_id}', 'pronamic_ideal' ),
	                'type'       => 'text',
	                'desc' => sprintf( __( 'Available tags: %s', 'pronamic_ideal' ), sprintf( '<code>%s</code>', '{order_id}' ) ),
	                'visibility' => array(
	                    'state'       => 'show',
	                    'conditional' => array(
	                        array(
	                            'field'   => '[enable]',
	                            'compare' => '=',
	                            'value'   => 'yes'
	                        ),
	                        array(
	                            'field'   => '[test_mode]',
	                            'compare' => '!=',
	                            'value'   => 'yes'
	                        )
	                    )
	                )
	            ),
	        )
	    );
	}
	
	/**
	 * Init.
	 */
	private function init() {
	    $this->config_id = $this->settings->get( 'config_id' );
	}
	
	public function process_payment( $order_id ) {
	    
	    $config_id = $this->config_id;
	    $payment_method = $this->id;
	    
	    // Use default gateway if no configuration has been set.
	    if ( '' === $config_id ) {
	        $config_id = get_option( 'pronamic_pay_config_id' );
	    }
	    
	    $gateway = Plugin::get_gateway( $config_id );
	    
	    if ( ! $gateway ) {
	        return false;
	    }
	    
	    // Data.
	    $data = new PaymentData($order_id,  $gateway, $this->settings );
	    
	    $gateway->set_payment_method( $payment_method );
	    
	    try {
	        $payment = Plugin::start( $config_id, $gateway, $data, $payment_method );
	        
	        $error = $gateway->get_error();
	        
	        if ( is_wp_error( $error ) ) {
	            throw new \Exception( $error->get_error_message() );
	        }
	        
	        return array(
	            'redirect' => $payment->get_pay_redirect_url(),
	            'result'   => $payment->get_pay_redirect_url() ? 'success' : 'fail',
	        );
	    } catch ( \Exception $e ) {
	        
	        learn_press_add_message( Plugin::get_default_error_message() . "<br>". $e->getMessage(), 'error' );
	        
	        return array(
	            'result' => 'fail'
	        );
	    }
	    
	    return array(
	        'result'   => 'fail'
	    );
	    
	}
}
