<?php

namespace KnitPay\Extensions\LearnPress;

use Pronamic\WordPress\Money\Money;
use Pronamic\WordPress\Money\TaxedMoney;
use Pronamic\WordPress\Money\Parser as MoneyParser;
use Pronamic\WordPress\Pay\Payments\PaymentLines;
use Pronamic\WordPress\Pay\Payments\PaymentData as Pay_PaymentData;
use Pronamic\WordPress\Pay\Payments\Item;
use Pronamic\WordPress\Pay\Payments\Items;

/**
 * Title: WordPress pay LearnPress payment data
 * Description:
 * Copyright: 2005-2020 Pronamic
 * Company: Pronamic
 *
 * @author  Remco Tolsma
 * @version 2.1.2
 * @since   1.0.0
 */
class PaymentData extends Pay_PaymentData {
	/**
	 * The order ID
	 *
	 * @var int
	 */
    private $order_id;

	/**
	 * Gateway
	 *
	 * @var Gateway
	 */
	private $gateway;

	/**
	 * User data
	 *
	 * @var array
	 */
	private $user_data;

	/**
	 * Constructs and initializes an LearnPress payment data object.
	 *
	 * @param Gateway            $gateway     gateway.
	 * @param int                        $order_id       Order ID.
	 */
	public function __construct( $order_id, $gateway, $settings ) {
	    $this->gateway =  $gateway;
	    $this->order_id = $order_id;
	    $this->settings = $settings;
	    
	    $this->order = learn_press_get_order( $order_id );
	    $this->user_id = $this->order->get_user_id();
	    $this->user_data = get_userdata($this->user_id );
	    
		parent::__construct();
	}

	/**
	 * Get source indicator
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_source()
	 * @return string
	 */
	public function get_source() {
		return 'learnpress';
	}

	/**
	 * Get source ID.
	 *
	 * @return string
	 */
	public function get_source_id() {
		return $this->order_id;
	}

	/**
	 * Get title.
	 *
	 * @return string
	 */
	public function get_title() {
		/* translators: %s: order id */
		return sprintf( __( 'Learn Press Order  %s', 'pronamic_ideal' ), $this->get_order_id() );
	}

	/**
	 * Get description
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_description()
	 * @return string
	 */
	public function get_description() {
	    $description = $this->settings->get( 'payment_description' );

		if ( '' === $description ) {
			$description = $this->get_title();
		}

		// Replacements.
		$replacements = array(
			'{order_id}' => $this->get_order_id(),
		);

		return strtr( $description, $replacements );
	}

	/**
	 * Get order ID
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_order_id()
	 * @return string
	 */
	public function get_order_id() {
		return $this->order_id;
	}

	/**
	 * Get items
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_items()
	 * @return Items
	 */
/* 	public function get_items() {
	       
	    $payment_lines = new PaymentLines();
	    if ( $items = LP()->get_cart()->get_items() ) {
	        foreach ( $items as $item ) {
	                
	                $line = $payment_lines->new_line();
	                
	                $line->set_id( $item['item_id'] );
	                $line->set_name( get_the_title( $item['item_id'] ) );
	                $line->set_quantity( $item['quantity'] );
	                $line->set_discount_amount( new Money($item['data']->get_sale_price(), $this->get_currency_alphabetic_code()) );
	                $line->set_unit_price( new TaxedMoney($item['data']->get_origin_price(), $this->get_currency_alphabetic_code()) );
	                $line->set_total_amount( new TaxedMoney($item['data']->get_sale_price(), $this->get_currency_alphabetic_code()) );
	                $line->set_product_url( get_permalink( $item['item_id'] ) );
	            }
	        }
	        
	        ppp($payment_lines);ddd();
	} */
	public function get_items() {
	    
	}
	
	public function get_amount(){
	    $amount = new TaxedMoney(learn_press_get_cart_total(),$this->get_currency_alphabetic_code());
	    return $amount;
	    
	    /* $total = 0;
	    if ( $items = LP()->get_cart()->get_items() ) {
	        foreach ( $items as $item ) {
	           $total += $item['total'];
	        }
	    }
	    $amount = new TaxedMoney($total,$this->get_currency_alphabetic_code());
	    return $amount; */
	}

	/**
	 * Get currency
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_currency_alphabetic_code()
	 * @return string
	 */
	public function get_currency_alphabetic_code() {
	    return learn_press_get_currency();
	}

	/**
	 * Get email.
	 *
	 * @return string
	 */
	public function get_email() {
	    return $this->user_data->user_email;
	}

	/**
	 * Get first name.
	 *
	 * @return string
	 */
	public function get_first_name() {
	    if ( isset( $this->user_data->first_name ) ) {
	        return $this->user_data->first_name;
		}
	}

	/**
	 * Get last name.
	 *
	 * @return string
	 */
	public function get_last_name() {
	    if ( isset( $this->user_data->last_name ) ) {
	        return $this->user_data->last_name;
	    }
	}

	/**
	 * Get customer name.
	 *
	 * @return string
	 */
	public function get_customer_name() {
	    return $this->user_data->first_name . ' ' . $this->user_data->last_name;
	}

	/**
	 * Get address.
	 *
	 * @return null|string
	 */
	/* public function get_address() {
		return $this->user_data['address'];
	} */

	/**
	 * Get city.
	 *
	 * @return null|string
	 */
	/* public function get_city() {
		return $this->user_data['city'];
	} */

	/**
	 * Get ZIP.
	 *
	 * @return null|string
	 */
	/* public function get_zip() {
		return $this->user_data['postcode'];
	} */

	/**
	 * Get telephone number.
	 *
	 * @return null|string
	 */
	public function get_telephone_number() {
	    return $this->user_data->phone;
	}

	/**
	 * Get normal return URL.
	 *
	 * @link https://github.com/woothemes/woocommerce/blob/v2.1.3/includes/abstracts/abstract-wc-payment-gateway.php#L52
	 * @return string
	 */
	public function get_normal_return_url() {ddd();
	    ppp(esc_url( $this->gateway->get_return_url( $this->order ) ));ddd();
	   return esc_url( $this->gateway->get_return_url( $this->order ) );
		
	    // TODO:
	    return charitable_get_permalink( 'donation_receipt_page', array( 'donation_id' => $this->donation_id ) );
	}

	/**
	 * Get cancel URL.
	 *
	 * @return string
	 */
	public function get_cancel_url() {ddd();
	    return esc_url( learn_press_is_enable_cart() ? learn_press_get_page_link( 'cart' ) : get_home_url() /* SITE_URL */ );
		$cancel_url = charitable_get_permalink( 'donation_cancel_page', array( 'donation_id' => $this->donation_id ) );

		if ( ! $cancel_url ) {
			$cancel_url = esc_url_raw(
				add_query_arg(
					array(
						'donation_id' => $this->donation_id,
						'cancel'      => true,
					),
					wp_get_referer()
				)
			);
		}

		return $cancel_url;
	}

	/**
	 * Get success URL.
	 *
	 * @return string
	 */
	public function get_success_url() {ddd();
	    return esc_url( $this->gateway->get_return_url( $this->order ) );
	}

	/**
	 * Get error URL.
	 *
	 * @return string
	 */
	public function get_error_url() {ddd();
	    return esc_url( learn_press_is_enable_cart() ? learn_press_get_page_link( 'cart' ) : get_home_url() /* SITE_URL */ );
		$cancel_url = charitable_get_permalink( 'donation_cancel_page', array( 'donation_id' => $this->donation_id ) );

		if ( ! $cancel_url ) {
			$cancel_url = esc_url_raw(
				add_query_arg(
					array(
						'donation_id' => $this->donation_id,
						'cancel'      => true,
					),
					wp_get_referer()
				)
			);
		}

		return $cancel_url;
	}
}
